package de.szut.lf8_starter.project.action.crud;

import de.szut.lf8_starter.employee.EmployeeService;
import de.szut.lf8_starter.project.ProjectEntity;
import de.szut.lf8_starter.project.ProjectService;
import de.szut.lf8_starter.project.dto.project.CreateProjectDto;
import de.szut.lf8_starter.project.dto.project.GetProjectDto;
import de.szut.lf8_starter.project.mapper.ProjectMapper;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import jakarta.validation.Valid;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping(value = "/projects")
public class CreateProjectAction {
    private final EmployeeService employeeService;
    private final ProjectService projectService;
    private final ProjectMapper projectMapper;

    public CreateProjectAction(EmployeeService employeeService, ProjectService projectService, ProjectMapper mappingService) {
        this.employeeService = employeeService;
        this.projectService = projectService;
        this.projectMapper = mappingService;
    }

    @Operation(summary = "Creates a new Project")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "201", description = "created project", content = {@Content(mediaType = "application/json", schema = @Schema(implementation = GetProjectDto.class))}),
            @ApiResponse(responseCode = "400", description = "invalid JSON posted", content = @Content),
            @ApiResponse(responseCode = "401", description = "not authorized", content = @Content),
            @ApiResponse(responseCode = "409", description = "Project dates conflict", content = @Content)
    })
    @PostMapping
    @ResponseStatus(code = HttpStatus.CREATED)
    public ResponseEntity<Object> create(
            @RequestBody @Valid CreateProjectDto createProjectDto,
            @RequestHeader("Authorization") String accessToken
    ) {
        ProjectEntity project = this.projectMapper.mapCreateDtoToEntity(createProjectDto);

        for (Long employeeId : createProjectDto.getEmployees()) {
            if (!this.employeeService.employeeExists(accessToken, employeeId)) {
                return new ResponseEntity<>("Employee with ID: " + employeeId + " not found", HttpStatus.NOT_FOUND);
            }

            for (GetProjectDto getProjectDto : this.employeeService.getProjects(employeeId)) {
                if (projectService.isOverlapping(getProjectDto, project)) {
                    return new ResponseEntity<>("Project dates conflict with an existing project for Employee with ID: " + employeeId, HttpStatus.CONFLICT);
                }
            }
        }

        this.projectService.create(project);

        return new ResponseEntity<>(this.projectMapper.mapToGetDto(project), HttpStatus.CREATED);
    }
}
