package de.szut.lf8_starter.integration.project;

import de.szut.lf8_starter.project.ProjectEntity;
import de.szut.lf8_starter.project.ProjectRepository;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.*;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
@DirtiesContext(classMode = DirtiesContext.ClassMode.AFTER_EACH_TEST_METHOD)
class AddEmployeeToProjectTest {
    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private ProjectRepository projectRepository;

    @Test
    void addEmployeeToProjectTest() throws Exception {
        ProjectEntity project = new ProjectEntity();
        project.setComment("comment");
        project.setContractor(1);
        project.setContractorName("contractorName");
        project.setEndDate(LocalDate.of(2024, 1, 1));
        project.setLeadingEmployee(1);
        project.setName("name");
        project.setStartDate(LocalDate.of(2021, 1, 1));
        project.setEmployees(List.of(1L, 2L, 3L));
        this.projectRepository.save(project);


        mockMvc.perform(post("/projects/{projectId}/employees/{employeeId}", 1, 312)
                        .header(HttpHeaders.AUTHORIZATION, getBearerToken())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNoContent());

        ProjectEntity updatedProject = projectRepository.findById(1L).get();

        assert updatedProject.getEmployees().contains(312L);
    }

    @Test
    void addEmployeeToPastProjectTest() throws Exception {
        ProjectEntity project = new ProjectEntity();
        project.setComment("past project");
        project.setContractor(1);
        project.setContractorName("contractorName");
        project.setPlannedEndDate(LocalDate.of(1991, 1, 1));
        project.setLeadingEmployee(1);
        project.setName("past project");
        project.setStartDate(LocalDate.of(1990, 1, 1));
        project.setEmployees(List.of(1L, 2L, 3L));
        this.projectRepository.save(project);

        mockMvc.perform(post("/projects/{projectId}/employees/{employeeId}", project.getId(), 312)
                        .header(HttpHeaders.AUTHORIZATION, getBearerToken())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNoContent());

        ProjectEntity updatedProject = projectRepository.findById(project.getId()).get();

        assert updatedProject.getEmployees().contains(312L);
    }

    @Test
    void addEmployeeToFutureProjectTest() throws Exception {
        ProjectEntity project = new ProjectEntity();
        project.setComment("future project");
        project.setContractor(1);
        project.setContractorName("contractorName");
        project.setPlannedEndDate(LocalDate.of(2101, 1, 1));
        project.setLeadingEmployee(1);
        project.setName("future project");
        project.setStartDate(LocalDate.of(2100, 1, 1));
        project.setEmployees(List.of(1L, 2L, 3L));
        this.projectRepository.save(project);

        mockMvc.perform(post("/projects/{projectId}/employees/{employeeId}", project.getId(), 312)
                        .header(HttpHeaders.AUTHORIZATION, getBearerToken())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNoContent());

        ProjectEntity updatedProject = projectRepository.findById(project.getId()).get();

        assert updatedProject.getEmployees().contains(312L);
    }

    @Test
    void addEmployeeToProjectWithOverlappingDatesTest() throws Exception {
        ProjectEntity project1 = new ProjectEntity();
        project1.setComment("project 1");
        project1.setContractor(1);
        project1.setContractorName("contractorName");
        project1.setPlannedEndDate(LocalDate.of(2023, 1, 1));
        project1.setLeadingEmployee(1);
        project1.setName("project 1");
        project1.setStartDate(LocalDate.of(2022, 1, 1));
        project1.setEmployees(List.of(1L, 2L, 3L));
        this.projectRepository.save(project1);

        ProjectEntity project2 = new ProjectEntity();
        project2.setComment("project 2");
        project2.setContractor(1);
        project2.setContractorName("contractorName");
        project2.setPlannedEndDate(LocalDate.of(2023, 6, 1));
        project2.setLeadingEmployee(1);
        project2.setName("project 2");
        project2.setStartDate(LocalDate.of(2022, 6, 1));
        project2.setEmployees(List.of(1L, 2L, 3L));
        this.projectRepository.save(project2);

        mockMvc.perform(post("/projects/{projectId}/employees/{employeeId}", project1.getId(), 312)
                        .header(HttpHeaders.AUTHORIZATION, getBearerToken())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNoContent());

        mockMvc.perform(post("/projects/{projectId}/employees/{employeeId}", project2.getId(), 312)
                        .header(HttpHeaders.AUTHORIZATION, getBearerToken())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isConflict());
    }

    private String getBearerToken() {
        String url = "https://keycloak.szut.dev/auth/realms/szut/protocol/openid-connect/token";

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);

        MultiValueMap<String, String> map = new LinkedMultiValueMap<>();
        map.add("grant_type", "password");
        map.add("client_id", "employee-management-service");
        map.add("username", "user");
        map.add("password", "test");

        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(map, headers);

        ResponseEntity<Map> response = this.restTemplate.exchange(url, HttpMethod.POST, request, Map.class);

        return Objects.requireNonNull(response.getBody()).get("access_token").toString();
    }
}
